% Monte Carlo simulation of 1-D diffusion from a point source
% A total of N particles is placed at a a single position at t=0.
% Each particle jumps with a probability W to the left or right with equal
% probabilities. Each partilce remains in its poistion over one timestep
% with the probabilty 1-W.

clear;

% Discretization
X=30;     % X:    number of points on x axis in the positive and negative directions
x=-X:X;   % x:    equally spaced points along x axis (vector of length 2X+1)
T=50;     % T:    nubmer of time steps/jumps  

% initialization 
N=1000;               % total number of particles
n=zeros(1,2*X+1);     % vector containing the particle numbers for each x
n(X+1)=N;             % the point source of N particles is placed at the X+1 node
m=zeros(1,2*X+1);     % auxiliary vector to store particle numbers after each time step

% loope over time, positions and particles
for i=1:T                 % loop over all time steps
    for j=2:(2*X)         % loop over all positions in x
        if n(j)>0
            for l=1:n(j)  % loop over all n(j) particles at position j
                r=rand;   % create a random number 0<rand<1
                if r < .5
                    m(j-1)=m(j-1)+1;                    
                else
                    m(j+1)=m(j+1)+1;
                end;
            end;
        end;
    end;
    m(1)=0;
    m(end)=0;
    n=m;
    m=zeros(1,2*X+1);
    plot(x,n,'.'), axis([-X X 0 2*N/10]); pause(0.1)
end;
hold on;
% Analysis of simulation results; comparison with Gaussian distribution

% fit Gaussian distribution to simulation result
% in a log(y) versus x^2 plot the Gaussian distribution is linear

% we have to take logarithms of n - before we can do that we must get rid
% of the 0 entries in vector n at the left and riight hand side of the 1D
% system.
% get rid of 0 in the vector n; map n into nf and x into xf adn get rd of
% the vector components nf(j) and xf(j), where n(j)=0
nf=n;
xf=x;
a=0;
for i=1:size(n,2)
    if nf(i)==0
       a(end+1)=i;        
    end;
end; 
a(1)=[];
nf(a)=[];
xf(a)=[];

% use polyfit(x,y,n), where n =1 for a linear fit
p=polyfit(xf.^2,log(nf),1);  % polyfit returns vector p with p(1): slope, p(2) intercept
z=exp(p(2))*exp(p(1)*x.^2);
%figure;
plot(x,n,'.',x,z); axis([-X X 0 2*N/10]); legend('rand','Gauss fit',0);
text(-.8*X, 1.1*max(n),['fitting = ',num2str(exp(p(2))),'*exp(',num2str(p(1)),' * x^2)']);
